@@ -1,4 +1,4 @@
-# $Id: Closures.pm,v 1.2 2004/09/30 06:10:16 comdog Exp $
+# $Id: Closures.pm,v 1.4 2005/01/26 18:21:25 comdog Exp $
 package File::Find::Closures;
 use strict;
 
@@ -6,10 +6,11 @@ use vars qw( $VERSION @EXPORT_OK %EXPORT_TAGS );
 
 use Carp qw(carp croak);
 use Exporter;
+use File::Basename qw(dirname);
 use File::Spec::Functions qw(canonpath no_upwards);
 use UNIVERSAL qw(isa);
 
-$VERSION = "0.010_01";
+$VERSION = "0.10_03";
 
 @EXPORT_OK   = ();
 %EXPORT_TAGS = ();
@@ -25,7 +26,7 @@ File::Find::Closures - functions you can use with File::Find
 	use File::Find;
 	use File::Find::Closures qw(:all);
 
-	my( $list_reporter, $wanted ) = find_by_name( qw(README) );
+	my( $wanted, $list_reporter ) = find_by_name( qw(README) );
 
 	File::Find::find( $wanted, @directories );
 	File::Find::find( { wanted => $wanted, ... }, @directories );
@@ -62,6 +63,20 @@ and the second one is the reporter.
 
 =over 4
 
+=item find_regular_files();
+
+Find all regular files.
+
+=cut
+
+sub find_regular_files
+	{	
+	my @files = ();
+	
+	sub { push @files, canonpath( $File::Find::name ) if -f $_ },
+	sub { @files = no_upwards( @files ); wantarray ? @files : [ @files ] }
+	}
+
 =item find_by_min_size( SIZE );
 
 Find files whose size is equal to or greater than SIZE bytes.
@@ -115,13 +130,14 @@ sub find_by_zero_size
 Find directories which contain files with the same name
 as any of the values in @names.
 
-UNIMPLEMENTED!
-
 =cut
 
 sub find_by_directory_contains
 	{
-	_unimplemented();
+	my @files = ();
+
+	sub { push @files, dirname( canonpath( $File::Find::name ) ) },
+	sub { wantarray ? @files : [ @files ] }
 	}
 
 =item find_by_name( @names );
@@ -175,13 +191,19 @@ sub find_by_regex
 Find files that are owned by the owner with the name OWNER_NAME.
 You can also use the owner's UID.
 
-UNIMPLEMENTED!
-
 =cut
 
 sub find_by_owner
 	{
-	_unimplemented();
+	my $id = getpwnam($_[0]);
+	   $id = $_ unless defined($id);
+
+	unless( $id =~ /\d+/ )
+		{
+		carp "Uid must be numeric of a valid system user name";
+		}
+
+	return _find_by_stat_part_equal( $id, 4 );
 	}
 
 =item find_by_group( GROUP_NAME | GROUP_GID );
@@ -189,13 +211,19 @@ sub find_by_owner
 Find files that are owned by the owner with the name GROUP_NAME.
 You can also use the group's GID.
 
-UNIMPLEMENTED!
-
 =cut
 
 sub find_by_group
 	{
-	_unimplemented();
+	my $id = getgrnam( $_[0] );
+	   $id = $_ unless defined( $id );
+
+	unless( $id =~ /\d+/ )
+		{
+		carp "Gid must be numeric or a valid system user name";
+		}
+
+	return _find_by_stat_part_equal( $id, 5 );
 	}
 
 =item find_by_executable();
@@ -204,13 +232,14 @@ Find files that are executable.  This may not work on some operating
 systems (like Windows) unless someone can provide me with an
 alternate version.
 
-UNIMPLEMENTED!
-
 =cut
 
 sub find_by_executable
 	{
-	_unimplemented();
+	my @files = ();
+	sub { push @files, canonpath( $File::Find::name )
+			if -x },
+	sub { wantarray ? @files : [ @files ] }
 	}
 
 =item find_by_writeable();
@@ -219,13 +248,14 @@ Find files that are writable.  This may not work on some operating
 systems (like Windows) unless someone can provide me with an
 alternate version.
 
-UNIMPLEMENTED!
-
 =cut
 
 sub find_by_writeable
 	{
-	_unimplemented();
+	my @files = ();
+	sub { push @files, canonpath( $File::Find::name )
+			if -w },
+	sub { wantarray ? @files : [ @files ] }
 	}
 
 =item find_by_umask( UMASK );
@@ -233,13 +263,17 @@ sub find_by_writeable
 Find files that fit the umask UMASK.  The files will not have those
 permissions.
 
-UNIMPLEMENTED!
-
 =cut
 
 sub find_by_umask
 	{
-	_unimplemented();
+	my ($mask) = @_;
+
+	my @files;
+
+	sub { push @files, canonpath( $File::Find::name )
+	       	if ((stat($_))[2] & $mask) == 0},
+	sub { wantarray ? @files : [ @files ] }
 	}
 
 =item find_by_modified_before( EPOCH_TIME );
@@ -248,13 +282,11 @@ Find files modified before EPOCH_TIME, which is in seconds since
 the local epoch (I may need to adjust this for some operating
 systems).
 
-UNIMPLEMENTED!
-
 =cut
 
 sub find_by_modified_before
 	{
-	_unimplemented();
+	return _find_by_stat_part_lessthan( $_[0], 9 );
 	}
 
 =item find_by_modified_after( EPOCH_TIME );
@@ -263,13 +295,11 @@ Find files modified after EPOCH_TIME, which is in seconds since
 the local epoch (I may need to adjust this for some operating
 systems).
 
-UNIMPLEMENTED!
-
 =cut
 
 sub find_by_modified_after
 	{
-	_unimplemented();
+	return _find_by_stat_part_greaterthan( $_[0], 9 );
 	}
 
 =item find_by_created_before( EPOCH_TIME );
@@ -278,13 +308,11 @@ Find files created before EPOCH_TIME, which is in seconds since
 the local epoch (I may need to adjust this for some operating
 systems).
 
-UNIMPLEMENTED!
-
 =cut
 
 sub find_by_created_before
 	{
-	_unimplemented();
+	return _find_by_stat_part_lessthan( $_[0], 10 );
 	}
 
 =item find_by_created_after( EPOCH_TIME );
@@ -293,13 +321,44 @@ Find files created after EPOCH_TIME, which is in seconds since
 the local epoch (I may need to adjust this for some operating
 systems).
 
-UNIMPLEMENTED!
-
 =cut
 
 sub find_by_created_after
 	{
-	_unimplemented();
+	return _find_by_stat_part_greaterthan( $_[0], 10 );
+	}
+
+sub _find_by_stat_part_equal 
+	{
+	my ($value, $stat_part) = @_;
+
+	my @files;
+
+	sub { push @files, canonpath( $File::Find::name )
+	       	if (stat($_))[$stat_part] == $value },
+	sub { wantarray ? @files : [ @files ] }
+	}
+
+sub _find_by_stat_part_lessthan 
+	{
+	my ($value, $stat_part) = @_;
+
+	my @files;
+
+	sub { push @files, canonpath( $File::Find::name )
+	       	if (stat($_))[$stat_part] < $value },
+	sub { wantarray ? @files : [ @files ] }
+	}
+
+sub _find_by_stat_part_greaterthan 
+	{
+	my ($value, $stat_part) = @_;
+
+	my @files;
+
+	sub { push @files, canonpath( $File::Find::name )
+	       	if (stat($_))[$stat_part] > $value },
+	sub { wantarray ? @files : [ @files ] }
 	}
 
 
@@ -334,7 +393,6 @@ The filename should be the full path to the file that you get
 from $File::Find::name, unless you are doing something wierd,
 like find_by_directory_contains().
 
-
 Once you have something, send it to me at C<< <bdfoy@cpan.org> >>. You
 must release your code under the Perl Artistic License.
 
@@ -366,6 +424,8 @@ members of the project can shepherd this module appropriately.
 
 brian d foy, C<< <bdfoy@cpan.org> >>
 
+Some functions implemented by Nathan Wagner, C<< <nw@hydaspes.if.org> >>
+
 =head1 COPYRIGHT
 
 Copyright (c) 2004, brian d foy, All Rights Reserved.
@@ -374,4 +434,4 @@ You may redistribute this under the same terms as Perl itself.
 
 =cut
 
-"Kanga and Baby Roo Come to the Forest";
+"Kanga and Baby Roo Come to the Forest";
\ No newline at end of file
